<?php
require_once plugin_dir_path(dirname(__FILE__)) . 'vendor/autoload.php';
use \Firebase\JWT\JWT;

class Api_Controller
{
    public function __construct()
    {
    }

    public function insert_order($mapping, $google_order_sheetid)
    {
        $SpreadsheetID  = sanitize_text_field($google_order_sheetid);
        $google_token   = get_option('wcgss_google_token', false);
        $credential     = get_option('wcgss_google_credential', false);
        if ($google_token  &&  time() > $google_token['expires_in']) {
            if ($credential) {
                $new_token = $this->wpgsi_token($credential);
                if ($new_token[0]) {
                    $new_token[1]['expires_in'] = time() + $new_token[1]['expires_in'];
                    $google_token = $new_token[1];
                    update_option('wcgss_google_token', $new_token[1]);
                }
            }
        }
        $google_token   = get_option('wcgss_google_token', false);
        $columnTitle    = $this->wcgss_columnTitle($SpreadsheetID, $google_token);
        $inserRecord  = $this->insert_record($mapping, $SpreadsheetID, $google_token, $columnTitle);
        $insertLogs = $this->wcgss_logs('order', $inserRecord);
        return $inserRecord;
    }

    public function insert_customer($mapping, $google_order_sheetid)
    {
        $SpreadsheetID  = sanitize_text_field($google_order_sheetid);
        $google_token   = get_option('wcgss_google_token', false);
        $credential     = get_option('wcgss_google_credential', false);

        if ($google_token  &&  time() > $google_token['expires_in']) {
            if ($credential) {
                $new_token = $this->wpgsi_token($credential);
                if ($new_token[0]) {
                    $new_token[1]['expires_in'] = time() + $new_token[1]['expires_in'];
                    $google_token = $new_token[1];
                    update_option('wcgss_google_token', $new_token[1]);
                }
            }
        }
        $columnTitle    = $this->wcgss_columnTitle($SpreadsheetID, $google_token);
        $inserRecord  = $this->insert_record($mapping, $SpreadsheetID, $google_token, $columnTitle);
        $insertLogs = $this->wcgss_logs('customer', $inserRecord);
        return $inserRecord;
    }

    public function updateInventory($columnTitle, $columnLists, $google_qty, $google_product_identifier)
    {
        $updateInventory = $this->update_inventory($columnTitle, $columnLists, $google_qty, $google_product_identifier);
        $log = json_encode($updateInventory);
        $insertLogs = $this->wcgss_logs('inventory', $log);
        return $updateInventory;
    }

    public function wcgss_columnTitle($spreadsheets_id = '', $token = [])
    {
        $request = wp_remote_get('https://sheets.googleapis.com/v4/spreadsheets/'. $spreadsheets_id . '/values/Sheet1!A1:YZ1?access_token='. $token['access_token']);

        if (is_wp_error($request)  or  ! isset($request['response']['code']) or $request['response']['code'] != 200) {
            return array( false, json_encode($request) );
        }


        $responseBody = json_decode($request['body'], true);

        if (!isset($responseBody['values'][0])) {
            return array( true, array( "A"=>"","B"=>"","C"=>"","D"=>"","E"=>"","F"=>"","G"=>"","H"=>"","I"=>"","J"=>"","K"=>"","L"=>"","M"=>"","N"=>"","O"=>"","P"=>"","Q"=>"","R"=>"","S"=>"","T"=>"","U"=>"","V"=>"","W"=>"","X"=>"","Y"=>"","Z"=>"" ) );
        }

        $key_array = array();
        for ($i = "A"; $i < 'ZZ' ; $i++) {
            array_push($key_array, $i);
        }
        $columnKeyTitle  = array_combine(array_slice($key_array, 0, count($responseBody['values'][0])), $responseBody['values'][0]);

        return$columnKeyTitle;
    }
    
    public function wcgss_column($spreadsheets_id = '', $token = [])
    {
        $request = wp_remote_get('https://sheets.googleapis.com/v4/spreadsheets/'. $spreadsheets_id . '/values/Sheet1!A1:Z100?access_token='. $token['access_token']);

        $responseBody = json_decode($request['body'], true);

        return $responseBody['values'];
    }

    public function insert_record($mapping, $spreadsheets_id, $token, $columnTitle)
    {
        foreach ($columnTitle as $key=>$value) {
            $mapped_val = str_replace(',', ' ', $mapping[$value]);
            $sheet_values[] = '"'.$mapped_val.'"';
        }

        $sheet_value = implode(', ', $sheet_values);

        if (! is_array($token)) {
            return array( false, "Error: Token is Not Array. from wpgsi_columnTitle func !"  );
        }
        if (! isset($token['access_token'])) {
            return array( false, "Error: access_token elements are not set on the token Array! from wpgsi_columnTitle func !"  );
        }
        if (empty($token['access_token'])) {
            return array( false, "Error: access_token elements is empty on the token Array! from wpgsi_columnTitle func !"  );
        }

        $postVal = '{"range": "Sheet1!A1:Z1","majorDimension": "ROWS","values": [['.$sheet_value.']]}';
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://sheets.googleapis.com/v4/spreadsheets/".$spreadsheets_id."/values/Sheet1!A1:Z1:append?valueInputOption=RAW&access_token=".$token['access_token']."",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $postVal,
            CURLOPT_HTTPHEADER => array(
                "cache-control: no-cache",
                "content-type: application/json",
                "postman-token: 41035b8f-242f-506e-fdfb-b39073f843dc"
            ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);
        curl_close($curl);
        return $response;
    }
    public function update_inventory($columnTitle, $columnLists, $google_qty, $google_product_identifier)
    {
        $sheet_Column_sku = array_search($google_product_identifier, $columnLists[0]);
        $sheet_Column_qty = array_search($google_qty, $columnLists[0]);

        foreach ($columnLists as $columnList) {
            $sku = $columnList[$sheet_Column_sku];
            $qty = $columnList[$sheet_Column_qty];
            $post_id = wc_get_product_id_by_sku($sku);
            update_post_meta($post_id, '_stock', $qty);
        }
        $data = array(
            'success' => true,
            'message' => 'Inentory Quantity Updated Successfully.',
    );

        return $data;
    }

    public function updateInventoryProductListing($columnLists,$sku)
    {
	    foreach($columnLists as $column)
	    {
		    if($column[0] == $sku)
		    {
			    $value = $column[1];
		    }
		    $post_id = wc_get_product_id_by_sku($sku);
		    update_post_meta($post_id, '_stock', $value);		    
	    }
	    $data = array(
            'success' => true,
            'message' => 'Inentory Quantity Updated Successfully.',
    );

        return $data;
    }

    public function update_order_notes($response, $order_status)
    {
        $sheet_id = $response['spreadsheetId'];
        $updatedRange = $response['updates']['updatedRange'];
        $google_token   = get_option('wcgss_google_token', false);
        $credential     = get_option('wcgss_google_credential', false);
        if ($google_token  &&  time() > $google_token['expires_in']) {
            if ($credential) {
                $new_token = $this->wpgsi_token($credential);
                if ($new_token[0]) {
                    $new_token[1]['expires_in'] = time() + $new_token[1]['expires_in'];
                    $google_token = $new_token[1];
                    update_option('wcgss_google_token', $new_token[1]);
                }
            }
        }

        $google_token   = get_option('wcgss_google_token', false);
        $updateOrder = $this->update_order_status($sheet_id, $updatedRange, $google_token, $order_status);
    }

    public function update_order_status($sheet_id, $updatedRange, $token, $order_status)
    {
        $columnTitle    = $this->wcgss_columnTitle($sheet_id, $token);
        if (! is_array($token)) {
            return array( false, "Error: Token is Not Array. from wpgsi_columnTitle func !"  );
        }
        if (! isset($token['access_token'])) {
            return array( false, "Error: access_token elements are not set on the token Array! from wpgsi_columnTitle func !"  );
        }
        if (empty($token['access_token'])) {
            return array( false, "Error: access_token elements is empty on the token Array! from wpgsi_columnTitle func !"  );
        }

        $updatedRange = "Sheet1!E11";
        $postVal = '{"range": "'.$updatedRange.'","majorDimension": "ROWS","values": [["'.$order_status.'"]]}';
        $curl = curl_init();
        curl_setopt_array($curl, array(
        CURLOPT_URL => "https://sheets.googleapis.com/v4/spreadsheets/".$sheet_id."/values/".$updatedRange."?valueInputOption=USER_ENTERED&access_token=".$token['access_token']."",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "PUT",
        CURLOPT_POSTFIELDS => $postVal,
        CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache",
            "content-type: application/json",
            "postman-token: 41035b8f-242f-506e-fdfb-b39073f843dc"
        ),
    ));

        $response = curl_exec($curl);
        $err = curl_error($curl);
        curl_close($curl);
        return $response;
    }
    public function wcgss_logs($module, $response)
    {
        global $wpdb;
        $tablename = $wpdb->prefix.'wcgss_logs';
        $wpdb->insert(
            $tablename,
            array(
            'module' => $module,
            'response' => $response),
            array( '%s', '%s' )
        );
    }
    public function wpgsi_token($credential = [])
    {
        if (! isset($credential['client_email'])) {
            return array( false, array('Error:'=> 420 , 'Message' => 'Error: client_email not set. from  wpgsi_token func !') );
        }
        if (empty($credential['client_email'])) {
            return array( false, array('Error:'=> 420 , 'Message' => "Error: client_email is Empty. from wpgsi_token func !") );
        }
        if (! isset($credential['private_key'])) {
            return array( false, array('Error:'=> 420 , 'Message' => "Error: private_key not set. from  wpgsi_token func !") );
        }
        if (empty($credential['private_key'])) {
            return array( false, array('Error:'=> 420 , 'Message' => "Error: private_key is Empty. from wpgsi_token func !") );
        }

        $payload = array(
            "iss"       =>  $credential['client_email'],
            "scope"     => 'https://www.googleapis.com/auth/drive',
            "aud"       => 'https://oauth2.googleapis.com/token',
            "exp"       =>      time()+3600,
            "iat"       =>      time(),
        );

        $jwt  = JWT::encode($payload, $credential['private_key'], 'RS256');

        $args = array(
            'headers' => array(),
            'body'    => array(
                'grant_type' => 'urn:ietf:params:oauth:grant-type:jwt-bearer',
                'assertion'  => $jwt,
            )
        );
        $returns  =  wp_remote_post('https://oauth2.googleapis.com/token', $args);
        if (is_wp_error($returns) or ! is_array($returns) or ! isset($returns['body'])) {
            return array( false, "Error :  on token Creation." . json_encode($returns, true)  );
        } else {
            return array( true, json_decode($returns['body'], true) );
        }
    }
    public function checkRole($userId)
    {
        if ($userId == 0) {
            return false;
        }
        $user = new WP_User($userId);
        $roles = $user->roles;
        if ($roles[0] == 'administrator') {
            return true;
        }
    }

    function checkOrderResquest($SpreadsheetID, $google_token)
{
        $columnTitle    = $this->wcgss_columnTitle($SpreadsheetID, $google_token);
        $wcgss_column = $this->wcgss_column($SpreadsheetID, $google_token);
        $orderArray = array();
        foreach ($columnTitle as $key=>$value) {
            if(isset($_POST['google_new_header'.$key]))
            {
                    $getColumn = $this->wcgss_columnTitle($SpreadsheetID, $google_token);
                    $lastKey = array_key_last($getColumn);
                    $nextColumn = ++$lastKey;
                    $update_header = $this->insert_header($SpreadsheetID, $google_token, $nextColumn, $_POST['google_new_header'.$key]);
                    $orderArray[$_POST['woo_order_attribute'.$key]] = $_POST['google_new_header'.$key];
            }
            else
            {
                    $orderArray[$_POST['woo_order_attribute'.$key]] = $_POST['sheet_header'.$key];
            }
	}
        $order_json =json_encode($orderArray, true);
	return $order_json;

}

function insert_header($spreadsheets_id = '', $token = [], $column, $columnHeaderName)
{

$curl = curl_init();

curl_setopt_array($curl, array(
  CURLOPT_URL => "https://sheets.googleapis.com/v4/spreadsheets/".$spreadsheets_id."/values/Sheet1!".$column."1:append?valueInputOption=USER_ENTERED",
  CURLOPT_RETURNTRANSFER => true,
  CURLOPT_ENCODING => "",
  CURLOPT_MAXREDIRS => 10,
  CURLOPT_TIMEOUT => 30,
  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
  CURLOPT_CUSTOMREQUEST => "POST",
  CURLOPT_POSTFIELDS => '{"values":[["'.$columnHeaderName.'"]]}',
  CURLOPT_HTTPHEADER => array(
    "accept: application/json",
    "authorization: Bearer ".$token['access_token']."",
    "cache-control: no-cache",
    "content-type: application/json",
    "postman-token: 0efa3e34-eca6-5279-3fec-80af4bdfadba"
  ),
));

$response = curl_exec($curl);
$err = curl_error($curl);

curl_close($curl);

return $response;

}

}
